@@ -3,7 +3,7 @@
  *
  * This file is part of mpop, a POP3 client.
  *
- * Copyright (C) 2000, 2003, 2004, 2005, 2006, 2007, 2008
+ * Copyright (C) 2000, 2003, 2004, 2005, 2006, 2007, 2008, 2009
  * Martin Lambers <marlam@marlam.de>
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -247,24 +247,24 @@ int get_non_neg_int(const char *s)
  * see conf.h
  */
 
-long get_size_arg(const char *s)
+long long get_size_arg(const char *s)
 {
-    long x;
+    long long x;
     char *p;
     
     errno = 0;
-    x = strtol(s, &p, 0);
-    if (p == s || x < 0 || (x == LONG_MAX && errno == ERANGE))
+    x = strtoll(s, &p, 0);
+    if (p == s || x < 0 || (x == LLONG_MAX && errno == ERANGE))
     {
 	x = -1;
     }    
     else if (strcmp(p, "k") == 0)
     {
-	x = (x > (LONG_MAX >> 10)) ? -1 : (x << 10);
+	x = (x > (LLONG_MAX >> 10)) ? -1 : (x << 10);
     }
     else if (strcmp(p, "m") == 0)
     {
-	x = (x > (LONG_MAX >> 20)) ? -1 : (x << 20);
+	x = (x > (LLONG_MAX >> 20)) ? -1 : (x << 20);
     }
     else if (*p != '\0')
     {
@@ -937,6 +937,14 @@ int read_conffile(const char *conffile, FILE *f, list_t **acc_list,
 		acc->delivery_args = expand_tilde(t);
 		free(t);
 	    }
+	    else if (strncmp(arg, "exchange", 8) == 0 && is_blank(arg[8]))
+	    {
+		acc->delivery_method = DELIVERY_METHOD_EXCHANGE;
+		free(acc->delivery_args);
+		t = trim_string(arg + 9);
+		acc->delivery_args = expand_tilde(t);
+		free(t);
+	    }
 	    else if (*arg == '\0')
 	    {	
 		*errstr = xasprintf(_("line %d: command %s needs an argument"),
