@@ -38,6 +38,10 @@
 #include <signal.h>
 #include <sys/wait.h>
 
+#if W32_NATIVE
+# include <share.h>
+#endif
+
 #include "gettext.h"
 #include "xalloc.h"
 #include "xvasprintf.h"
@@ -131,8 +135,8 @@ static void mda_sigpipe_handler(int signum UNUSED)
     mda_caused_sigpipe = 1;
 }
 
-int delivery_method_mda_open(delivery_method_t *dm, const char *from, long size,
-	char **errstr)
+int delivery_method_mda_open(delivery_method_t *dm, const char *from,
+	long long size, char **errstr)
 {
     int e;
     char *cmd;
@@ -145,7 +149,7 @@ int delivery_method_mda_open(delivery_method_t *dm, const char *from, long size,
     }
     if (dm->want_size)
     {
-	sizestr = xasprintf("%ld", size);
+	sizestr = xasprintf("%lld", size);
 	cmd = string_replace(cmd, "%S", sizestr);
 	free(sizestr);
     }
@@ -210,6 +214,7 @@ int delivery_method_mda_init(delivery_method_t *dm, void *data,
 
     dm->data = data;
     dm->need_from_quoting = 0;
+    dm->need_crlf = 0;
     dm->want_from_addr = (strstr((char *)data, "%F") != NULL);
     dm->want_size = 0;
     dm->open = delivery_method_mda_open;
@@ -318,7 +323,7 @@ typedef struct
 } maildir_data_t;
 
 int delivery_method_maildir_open(delivery_method_t *dm, const char *from UNUSED,
-	long size UNUSED, char **errstr)
+	long long size UNUSED, char **errstr)
 {
     maildir_data_t *maildir_data;
     char *filename;
@@ -334,9 +339,9 @@ int delivery_method_maildir_open(delivery_method_t *dm, const char *from UNUSED,
     }
     /* See http://cr.yp.to/proto/maildir.html for a description of file name
      * generation. */
-    filename = xasprintf("tmp%c%lu.M%06luP%ldQ%lu.%s", PATH_SEP,
-		(unsigned long)tv.tv_sec, (unsigned long)tv.tv_usec,
-		(long)getpid(), ++maildir_sequence_number, 
+    filename = xasprintf("tmp%c%llu.M%06luP%lldQ%lu.%s", PATH_SEP,
+		(unsigned long long)tv.tv_sec, (unsigned long)tv.tv_usec,
+		(long long)getpid(), ++maildir_sequence_number, 
 		maildir_data->hostname);
     /* Instead of waiting for stat() to return ENOENT, we open() the file with
      * O_CREAT | O_EXCL. There is no point in trying again after some time,
@@ -405,7 +410,7 @@ int delivery_method_maildir_init(delivery_method_t *dm, void *data,
     maildir_data = xmalloc(sizeof(maildir_data_t));
     maildir_data->maildir = xstrdup((char *)data);
     maildir_data->filename = NULL;
-    if (gethostname(hostname, 256) != 0)
+    if (gethostname(hostname, 256) != 0 || hostname[0] == '\0')
     {
 	/* Should never happen on any sane system */
 	strcpy(hostname, "unknown");
@@ -426,6 +431,7 @@ int delivery_method_maildir_init(delivery_method_t *dm, void *data,
 	    "_134_");
     dm->data = maildir_data;
     dm->need_from_quoting = 0;
+    dm->need_crlf = 0;
     dm->want_from_addr = 0;
     dm->want_size = 0;
     dm->open = delivery_method_maildir_open;
@@ -452,6 +458,163 @@ int delivery_method_maildir_deinit(delivery_method_t *dm, char **errstr UNUSED)
 }
 
 
+/*******************************************************************************
+ *
+ *  The MS exchange method.
+ *  
+ *  This is similar to the maildir code.
+ *
+ ******************************************************************************/
+
+/* This number is unique for the current process. It is used to create unique
+ * exchange filenames. */
+static unsigned long exchange_sequence_number = 0;
+
+typedef struct
+{
+    char *pickupdir;
+    char *filename;
+    char *hostname;
+} exchange_data_t;
+
+int delivery_method_exchange_open(delivery_method_t *dm, const char *from UNUSED,
+	long long size UNUSED, char **errstr)
+{
+    exchange_data_t *exchange_data;
+    char *filename;
+    struct timeval tv;
+    int fd;
+
+    exchange_data = dm->data;
+    if (gettimeofday(&tv, NULL) < 0)
+    {
+	*errstr = xasprintf(_("cannot get system time: %s"), strerror(errno));
+	return DELIVERY_EUNKNOWN;
+    }
+    /* Choose a unique filename (similar to the maildir method) that ends with
+     * ".eml" */
+    filename = xasprintf("%s-%llu-M%06luP%lldQ%lu-%s.eml", PACKAGE_NAME,
+		(unsigned long long)tv.tv_sec, (unsigned long)tv.tv_usec,
+		(long long)getpid(), ++exchange_sequence_number, 
+		exchange_data->hostname);
+#if W32_NATIVE
+    /* Open the file and deny read and write access to other processes, e.g.
+     * Exchange */
+    if ((fd = _sopen(filename, O_WRONLY | O_CREAT | O_EXCL, _SH_DENYRW, 
+		    S_IRUSR | S_IWUSR)) < 0)
+#else
+    /* We cannot do the same on UNIX; just open the file normally */
+    if ((fd = open(filename, O_WRONLY | O_CREAT | O_EXCL, 
+		    S_IRUSR | S_IWUSR)) < 0)
+#endif
+    {
+	*errstr = xasprintf(_("cannot create %s%c%s: %s"), 
+		exchange_data->pickupdir, PATH_SEP, filename, strerror(errno));
+	free(filename);
+	return DELIVERY_EIO;
+    }
+    exchange_data->filename = filename;
+    if (!(dm->pipe = fdopen(fd, "w")))
+    {
+	*errstr = xasprintf(_("cannot open %s%c%s: %s"), 
+		exchange_data->pickupdir, PATH_SEP, exchange_data->filename,
+		strerror(errno));
+	return DELIVERY_EIO;
+    }
+    /* The locking is superfluous (but harmless) on W32 because the file is
+     * opened with sharing disabled. It is mostly done for UNIX systems to at
+     * least have some kind of protection against processes reading half-written
+     * files (of course, they would have to obey locking for this). */
+    if (lock_file(dm->pipe, TOOLS_LOCK_WRITE, 0) != 0)
+    {
+	*errstr = xasprintf(_("cannot lock %s%c%s: %s"), 
+		exchange_data->pickupdir, PATH_SEP, exchange_data->filename,
+		strerror(errno));
+	return DELIVERY_EIO;
+    }
+    return DELIVERY_EOK;
+}
+
+int delivery_method_exchange_close(delivery_method_t *dm, char **errstr)
+{
+    exchange_data_t *exchange_data;
+
+    exchange_data = dm->data;
+    if (fsync(fileno(dm->pipe)) != 0)
+    {
+	*errstr = xasprintf(_("cannot sync %s%c%s: %s"), exchange_data->pickupdir, 
+		PATH_SEP, exchange_data->filename, strerror(errno));
+	return DELIVERY_EIO;
+    }
+    if (fclose(dm->pipe) != 0)
+    {
+	*errstr = xasprintf(_("cannot close %s%c%s: %s"), exchange_data->pickupdir,
+		PATH_SEP, exchange_data->filename, strerror(errno));
+	return DELIVERY_EIO;
+    }
+    free(exchange_data->filename);
+    exchange_data->filename = NULL;
+
+    return DELIVERY_EOK;
+}
+
+int delivery_method_exchange_init(delivery_method_t *dm, void *data, 
+	char **errstr)
+{
+    exchange_data_t *exchange_data;
+    char hostname[256];
+    
+    exchange_data = xmalloc(sizeof(exchange_data_t));
+    exchange_data->pickupdir = xstrdup((char *)data);
+    exchange_data->filename = NULL;
+    if (gethostname(hostname, 256) != 0 || hostname[0] == '\0')
+    {
+	/* Should never happen on any sane system */
+	strcpy(hostname, "unknown");
+    }
+    else
+    {
+	/* Make sure the hostname is NUL-terminated. */
+	hostname[255] = '\0';
+    }
+    exchange_data->hostname = xstrdup(hostname);
+    /* replace invalid characters as described in
+     * http://cr.yp.to/proto/maildir.html */
+    exchange_data->hostname = string_replace(exchange_data->hostname, "/", 
+	    "_057_");
+    exchange_data->hostname = string_replace(exchange_data->hostname, ":", 
+	    "_072_");
+    exchange_data->hostname = string_replace(exchange_data->hostname, "\\", 
+	    "_134_");
+    dm->data = exchange_data;
+    dm->need_from_quoting = 0;
+    dm->need_crlf = 1;
+    dm->want_from_addr = 0;
+    dm->want_size = 0;
+    dm->open = delivery_method_exchange_open;
+    dm->close = delivery_method_exchange_close;
+    if (chdir(exchange_data->pickupdir) != 0)
+    {
+	*errstr = xasprintf(_("cannot change to %s: %s"), exchange_data->pickupdir,
+		strerror(errno));
+	return DELIVERY_EUNKNOWN;
+    }
+    (void)umask(S_IRWXG | S_IRWXO);
+
+    return DELIVERY_EOK;
+}
+
+int delivery_method_exchange_deinit(delivery_method_t *dm, char **errstr UNUSED)
+{
+    exchange_data_t *exchange_data = dm->data;
+    free(exchange_data->pickupdir);
+    free(exchange_data->filename);
+    free(exchange_data->hostname);
+    free(exchange_data);
+    return DELIVERY_EOK;
+}
+
+
 /*******************************************************************************
  *
  *  The mbox method
@@ -459,7 +622,7 @@ int delivery_method_maildir_deinit(delivery_method_t *dm, char **errstr UNUSED)
  ******************************************************************************/
 
 int delivery_method_mbox_open(delivery_method_t *dm, const char *from, 
-	long size UNUSED, char **errstr)
+	long long size UNUSED, char **errstr)
 {
     time_t t;
 
@@ -512,6 +675,7 @@ int delivery_method_mbox_init(delivery_method_t *dm, void *data, char **errstr)
 
     dm->data = data;
     dm->need_from_quoting = 1;
+    dm->need_crlf = 0;
     dm->want_from_addr = 1;
     dm->want_size = 0;
     dm->open = delivery_method_mbox_open;
@@ -589,6 +753,10 @@ delivery_method_t *delivery_method_new(int method, void *data, char **errstr)
 	    e = delivery_method_mbox_init(dm, data, errstr);
 	    break;
 
+	case DELIVERY_METHOD_EXCHANGE:
+	    e = delivery_method_exchange_init(dm, data, errstr);
+	    break;
+
 	case DELIVERY_METHOD_FILTER:
 	    e = delivery_method_filter_init(dm, data, errstr);
 	    break;
@@ -629,6 +797,10 @@ int delivery_method_free(delivery_method_t *dm, char **errstr)
 	    e = delivery_method_mbox_deinit(dm, errstr);
 	    break;
 
+	case DELIVERY_METHOD_EXCHANGE:
+	    e = delivery_method_exchange_deinit(dm, errstr);
+	    break;
+
 	case DELIVERY_METHOD_FILTER:
 	    e = delivery_method_filter_deinit(dm, errstr);
 	    break;
