@@ -817,25 +817,25 @@ error_exit:
  * Returns a pointer to this string.
  */
 
-char *mpop_hr_size(long size)
+char *mpop_hr_size(long long size)
 {
     char *s;
     
     if (size >= 1024 * 1024 * 1024)
     {
-	s = xasprintf(_("%.2f GB"), (float)size / (float)(1024 * 1024 * 1024));
+	s = xasprintf(_("%.2f GiB"), (float)size / (float)(1024 * 1024 * 1024));
     }
     else if (size >= 1024 * 1024)
     {
-	s = xasprintf(_("%.2f MB"), (float)size / (float)(1024 * 1024));
+	s = xasprintf(_("%.2f MiB"), (float)size / (float)(1024 * 1024));
     }
     else if (size >= 1024)
     {
-	s = xasprintf(_("%.2f KB"), (float)size / 1024.0);
+	s = xasprintf(_("%.2f KiB"), (float)size / 1024.0f);
     }
     else if (size > 1 || size == 0)
     {
-	s = xasprintf(_("%ld bytes"), size);
+	s = xasprintf(_("%lld bytes"), size);
     }
     else
     {
@@ -871,7 +871,7 @@ void mpop_filter_output(long i, long number, int new_action, void *data)
     }
 }
 
-void mpop_retr_progress_start(long i, long number, long size)
+void mpop_retr_progress_start(long i, long number, long long size)
 {
     char *sizestr = mpop_hr_size(size);
     printf(_("retrieving message %ld of %ld (%s): "), 
@@ -881,20 +881,21 @@ void mpop_retr_progress_start(long i, long number, long size)
     fflush(stdout);
 }
 
-void mpop_retr_progress(long i UNUSED, long number UNUSED, long rcvd UNUSED,
-	long size UNUSED, int percent)
+void mpop_retr_progress(long i UNUSED, long number UNUSED,
+	long long rcvd UNUSED, long long size UNUSED, int percent)
 {
     printf("%3d\b\b\b", percent);
     fflush(stdout);
 }
 
-void mpop_retr_progress_end(long i UNUSED, long number UNUSED, long size UNUSED)
+void mpop_retr_progress_end(long i UNUSED, long number UNUSED,
+	long long size UNUSED)
 {
     printf("100\n");
 }
 
 void mpop_retr_progress_abort(long i UNUSED, long number UNUSED, 
-	long size UNUSED)
+	long long size UNUSED)
 {
     printf("\n");
 }
@@ -1534,12 +1535,21 @@ int make_needed_dirs(const char *pathname)
     error = 0;
     dir_part_end = strchr(pathname + 1, PATH_SEP);
 #if W32_NATIVE
-    /* skip a drive letter */
     if (dir_part_end - pathname == 2 
 	    && c_isalpha((unsigned char)pathname[0]) && pathname[1] == ':')
     {
+        /* skip drive letter ("C:\" and similar) */
 	dir_part_end = strchr(dir_part_end + 1, PATH_SEP);
     }
+    else if (dir_part_end - pathname == 1)
+    {
+        /* skip network resource name ("\\server\" and similar) */
+        dir_part_end = strchr(dir_part_end + 1, PATH_SEP);
+        if (dir_part_end)
+        {
+            dir_part_end = strchr(dir_part_end + 1, PATH_SEP);
+        }
+    }
 #endif
     while (dir_part_end && !error)
     {
@@ -2112,6 +2122,12 @@ int main(int argc, char *argv[])
 		    free(cmdline_account->delivery_args);
 		    cmdline_account->delivery_args = expand_tilde(optarg + 5);
 		}
+		else if (strncmp(optarg, "exchange,", 9) == 0)
+		{
+		    cmdline_account->delivery_method = DELIVERY_METHOD_EXCHANGE;
+		    free(cmdline_account->delivery_args);
+		    cmdline_account->delivery_args = expand_tilde(optarg + 9);
+		}
 		else
 		{
 		    print_error(_("invalid argument %s for %s"), 
@@ -2536,6 +2552,10 @@ int main(int argc, char *argv[])
 		{
 		    printf("mbox");
 		}
+		else if (account->delivery_method == DELIVERY_METHOD_EXCHANGE)
+		{
+		    printf("exchange");
+		}
 		printf(" %s\n", account->delivery_args);
 		printf("uidls file            = %s\n"
 			"only_new              = %s\n"
@@ -2550,7 +2570,7 @@ int main(int argc, char *argv[])
 		}
 		else
 		{
-		    printf("%ld\n", account->killsize);
+		    printf("%lld\n", account->killsize);
 		}
 		printf("skipsize              = ");
 		if (account->skipsize < 0)
@@ -2559,7 +2579,7 @@ int main(int argc, char *argv[])
 		}
 		else
 		{
-		    printf("%ld\n", account->skipsize);
+		    printf("%lld\n", account->skipsize);
 		}
 		printf("filter                = %s\n",
 			account->filter ? account->filter : _("(not set)"));
