@@ -365,22 +365,29 @@ int uidls_read(const char *filename, FILE **uidls_file, list_t **uidl_list,
 int uidls_write(const char *filename, FILE *uidls_file, list_t *uidl_list, 
 	char **errstr)
 {
+    int temp_fd;
+    char *temp_filename;
+    FILE *temp_file;
     list_t *lp;
     uidl_t *uidl;
     long i;
     int error;
 
-    if (fseeko(uidls_file, 0, SEEK_SET) != 0 
-	    || ftruncate(fileno(uidls_file), 0) != 0)
+    temp_filename = xmalloc((strlen(filename) + 7 + 1) * sizeof(char));
+    strcpy(temp_filename, filename);
+    strcat(temp_filename, ".XXXXXX");
+    if ((temp_fd = mkstemp(temp_filename)) == -1
+	    || !(temp_file = fdopen(temp_fd, "w")))
     {
 	if (errstr)
 	{
-	    *errstr = xasprintf("%s: %s", filename, strerror(errno));
+	    *errstr = xasprintf("%s: %s", temp_filename, strerror(errno));
 	}
-	return UIDLS_EOK;
+	free(temp_filename);
+	fclose(uidls_file);
+	return UIDLS_EIO;
     }
-
-    error = (fprintf(uidls_file, 
+    error = (fprintf(temp_file, 
 		"# This file was generated by %s version %s.\n"
 		"#\n"
 		"# Lines starting with a space begin a new UID list and have\n"
@@ -389,7 +396,6 @@ int uidls_write(const char *filename, FILE *uidls_file, list_t *uidl_list,
 			"UID per line.\n"
 		"#\n",
 		PACKAGE_NAME, PACKAGE_VERSION) < 0);
-    
     lp = uidl_list;
     while (!error && !list_is_empty(lp))
     {
@@ -397,33 +403,58 @@ int uidls_write(const char *filename, FILE *uidls_file, list_t *uidl_list,
 	uidl = lp->data;
 	if (uidl->n > 0)
 	{
-	    error = (fprintf(uidls_file, " %ld %s %s\n", 
+	    error = (fprintf(temp_file, " %ld %s %s\n", 
 			uidl->n, uidl->hostname, uidl->user) < 0);
 	    qsort(uidl->uidv, (size_t)uidl->n, sizeof(char *), uidls_uidcmp);
 	    for (i = 0; !error && i < uidl->n; i++)
 	    {
-		error = (fputs(uidl->uidv[i], uidls_file) == EOF 
-			|| fputc((unsigned char)'\n', uidls_file) == EOF);
+		error = (fputs(uidl->uidv[i], temp_file) == EOF 
+			|| fputc((unsigned char)'\n', temp_file) == EOF);
 	    }
 	}
     }
-    if (!error)
-    {
-	error = (fsync(fileno(uidls_file)) != 0);
-    }
-
     if (error)
     {
 	if (errstr)
 	{
-	    *errstr = xasprintf("%s: %s", filename, strerror(errno));
+	    *errstr = xasprintf(_("%s: output error"), temp_filename);
 	}
-	(void)fclose(uidls_file);
+	free(temp_filename);
+	fclose(temp_file);
+	fclose(uidls_file);
 	return UIDLS_EIO;
     }
-    else
+    if (fflush(temp_file) != 0 || fsync(temp_fd) != 0 || fclose(temp_file) != 0)
     {
-	(void)fclose(uidls_file);
-	return UIDLS_EOK;
+	if (errstr)
+	{
+	    *errstr = xasprintf("%s: %s", temp_filename, strerror(errno));
+	}
+	free(temp_filename);
+	fclose(temp_file);
+	fclose(uidls_file);
+	return UIDLS_EIO;
     }
+#ifdef W32_NATIVE
+    /* Even with the gnulib rename module, W32 cannot perform the rename if the
+     * destination file is opened by us. We need to close it first. This opens a
+     * window in which another mpop process might start to use the file... */
+    fclose(uidls_file);
+#endif
+    if (rename(temp_filename, filename) != 0)
+    {
+	if (errstr)
+	{
+	    *errstr = xasprintf(_("cannot rename %s to %s: %s"),
+		    temp_filename, filename, strerror(errno));
+	}
+	free(temp_filename);
+	fclose(uidls_file);
+	return UIDLS_EIO;
+    }
+    free(temp_filename);
+#ifndef W32_NATIVE
+    fclose(uidls_file);
+#endif
+    return UIDLS_EOK;
 }
